import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';
import 'package:url_launcher/url_launcher.dart';

typedef OnLocationUpdateCallback = void Function(String url);
typedef OnOauthOpenCallback = void Function(String oauthUrl);
typedef OnWebViewCreatedCallback = void Function(
    InAppWebViewController controller);
typedef OnWebViewLoadStartCallback = void Function();

class ConnectWebView extends StatelessWidget {
  const ConnectWebView(
      {Key? key,
      required this.url,
      required this.onLocationUpdate,
      required this.onOauthOpen,
      required this.onWebViewCreated,
      required this.onWebViewLoadStart})
      : super(key: key);

  final String url;
  final OnLocationUpdateCallback onLocationUpdate;
  final OnOauthOpenCallback onOauthOpen;
  final OnWebViewCreatedCallback onWebViewCreated;
  final OnWebViewLoadStartCallback onWebViewLoadStart;

  void _onWebViewCreated(InAppWebViewController controller) {
    onWebViewCreated(controller);

    controller.addJavaScriptHandler(
        handlerName: 'pluggyConnectHandler',
        callback: (args) {
          dynamic dataPayload;

          try {
            dataPayload = jsonDecode(args[0] as String);
          } catch (error) {
            return;
          }

          final String type = dataPayload['type'];
          final String message = dataPayload['message'] ?? '';

          if (type == 'OAUTH_OPEN') {
            if (!message.startsWith('https')) {
              // in the of open an intent or any other type of url that it's not http, we try to open outside the app,
              // since webview doesn't support it
              _openLink(message);
              return;
            }
            onOauthOpen(message);
            return;
          }

          if (type == 'LOCATION') {
            // got a location URL update
            onLocationUpdate(message);
            return;
          }

          if (type == 'LINK_OPEN') {
            // got a link URL open it outside the app
            _openLink(message);
            return;
          }
        });
  }

  /// Open a link outside the app
  void _openLink(String url) async {
    Uri parsedUri = Uri.parse(url);
    if (await canLaunchUrl(parsedUri)) {
      await launchUrl(parsedUri, mode: LaunchMode.externalApplication);
    } else {
      // can't open URL, do nothing (this should never happen), if happens at least not bubble up
    }
  }

  void _onLoadStart(InAppWebViewController controller, Uri? uri) {
    onWebViewLoadStart();
  }

  void _onLoadStop(InAppWebViewController controller, Uri? uri) async {
    /* Note: We are injecting 'ReactNativeWebView' variable because it's implemented
        like this in Connect webapp. We'll keep it like this to simplify API in there. */
    await controller.evaluateJavascript(source: '''
      window.ReactNativeWebView = {
        postMessage: function(message) {
          window.flutter_inappwebview.callHandler("pluggyConnectHandler", message);
        }
      };
      true; // note: this is required, or you'll sometimes get silent failures
    ''');
  }

  @override
  Widget build(BuildContext context) {
    /// This is required to avoid the webview to block the gestures of the parent widget
    var gestureRecognizers = <Factory<OneSequenceGestureRecognizer>>[
      Factory<OneSequenceGestureRecognizer>(
        () => EagerGestureRecognizer(),
      ),
    ].toSet();
    return SafeArea(
        child: InAppWebView(
      gestureRecognizers: gestureRecognizers,
      initialUrlRequest: URLRequest(url: Uri.parse(url)),
      onWebViewCreated: _onWebViewCreated,
      onLoadStart: _onLoadStart,
      onLoadStop: _onLoadStop,
    ));
  }
}
